/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#ifndef _DERIVED_METRICS_COLLECTION_CPP
#define _DERIVED_METRICS_COLLECTION_CPP

#include "DerivedMetricsCollection.h"

#define EMPTY_METRIC "metric type: \n" \
    "display name: \n" \
    "unique name: \n" \
    "uom: \n" \
    "description: \n" \
    "cubepl expression: \n"


#define KENOBI_METRIC "metric type: postderived\n" \
    "display name: Average visit time\n" \
    "unique name: avg_visit_time\n" \
    "uom: sec\n" \
    "description: Calculates average time of region execution per visit.\n" \
    "cubepl expression: metric::time(i)/metric::visits(e)\n"

#define REGION_SELECTION_METRIC "metric type: postderived\n" \
    "display name: Region __EDIT__\n" \
    "unique name: region_name_selection\n" \
    "uom: occ\n" \
    "description: Returns 1 in for the reg.\n" \
    "cubepl expression: { \n" \
    " ${a}=0;\n" \
    "  if ( ${cube::region::name}[${calculation::region::id}] =~  /^__EDIT__/ ) \n" \
    "     { ${a}=1; }; return ${a}; \n" \
    " }\n"

#define LEAVES_SELECTION_METRIC "metric type: prederived_exclusive\n" \
    "display name: Leaves selection\n" \
    "unique name: leaves_selection\n" \
    "uom: occ\n" \
    "description: Returns 1 for the callpath is a leaf.\n" \
    "cubepl expression: { \n" \
    " ${a}=0;\n" \
    "  if (${cube::callpath::#children}[${calculation::callpath::id}] == 0 )\n" \
    "     { ${a}=1; }; return ${a}; \n" \
    " }\n"

#define ROOT_SELECTION_METRIC "metric type: prederived_exclusive\n" \
    "display name: Root selection\n" \
    "unique name: root_selection\n" \
    "uom: occ\n" \
    "description: Returns 1 for the root callpath.\n" \
    "cubepl expression: { \n" \
    " ${a}=0;\n" \
    " if (${cube::callpath::parent::id}[${calculation::callpath::id}] == -1 )\n" \
    "     { ${a}=1; }; return ${a}; \n" \
    " }\n"


#define N_LEVEL_SELECTION_METRIC "metric type: prederived_exclusive\n" \
    "display name: Selection Level 1\n" \
    "unique name: selection_level_1\n" \
    "uom: occ\n" \
    "description: Returns 1 for the callpath of level 1.\n" \
    "To change the level of callpath one modifies variable \"level\".\n" \
    "cubepl expression: { \n" \
    "${level}=1;\n" \
    "${index}=0;\n" \
    "${i}=${calculation::callpath::id};\n\n" \
    "while (${cube::callpath::parent::id}[${i}] != -1 )\n" \
    "{ \n" \
    "  ${i}= ${cube::callpath::parent::id}[${i}]; \n" \
    "  ${index}=${index}+1; \n" \
    "}; \n" \
    "  ${a}=0;\n" \
    "  if (${index} == ${level})\n" \
    "  {  ${a}=1; };\n" \
    "return ${a}; \n" \
    "}  "


QString&
DerivedMetricsCollection::getDerivedMetricText( int key )
{
    if ( key >= NUM_DERIVED_METRICS )
    {
        return derived_metrics[ DN_NO_METRIC ];
    }
    return derived_metrics[ key ];
}



QString DerivedMetricsCollection::derived_metrics[ NUM_DERIVED_METRICS ] = {
    QString( EMPTY_METRIC ),
    QString( KENOBI_METRIC ),
    QString( REGION_SELECTION_METRIC ),
    QString( LEAVES_SELECTION_METRIC ),
    QString( ROOT_SELECTION_METRIC ),
    QString( N_LEVEL_SELECTION_METRIC )
};


QString DerivedMetricsCollection::derived_metrics_names[ NUM_DERIVED_METRICS ] = {
    QString( "--- please select ---" ),
    QString( "Average execution time (kenobi)" ),
    QString( "Region selection" ),
    QString( "Leaf selection" ),
    QString( "Root selecton metric" ),
    QString( "#Level selection metric" )
};


QStringList
DerivedMetricsCollection::getDerivedMetricsNames()
{
    QStringList _to_return;
    for ( unsigned i = 0; i < NUM_DERIVED_METRICS; i++ )
    {
        _to_return <<  derived_metrics_names[ i ];
    }
    return _to_return;
}

#endif
