/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#include <QThread>
#include <QDialogButtonBox>
#include <QMessageBox>
#include <QFileDialog>



#include "MainWidget.h"
#include "MessageWidget.h"
#include "Statistics.h"

#include <cassert>
#include <iostream>

#include "Action.h"

#include <QDialogButtonBox>
#include <QMessageBox>
#include <QFileDialog>

using namespace std;

#if defined( WITH_VAMPIR )
#include "VampirConnectionDialog.h"
#endif





void
MainWidget::createTraceBrowserMenu()
{
    traceBrowserMenu = 0;
#if defined( WITH_VAMPIR ) || defined( WITH_PARAVER )
    traceBrowserMenu = fileMenu->addMenu( "Connect to trace browser" );
    traceBrowserMenu->setStatusTip( tr( "Ready" ) );
    traceBrowserMenu->setEnabled( false );
    traceBrowserMenu->menuAction()->setVisible( false );
#endif

#if defined( WITH_VAMPIR )
    vampirAct = 0;
    vampirAct = new Action( tr( "Connect to &vampir..." ), this );
    vampirAct->setStatusTip( tr( "Connect to vampir and display a trace file" ) );
    vampirAct->setWhatsThis( "Connect to vampir and display a trace file" );
    connect( vampirAct, SIGNAL( triggered() ), this, SLOT( vampirMenu() ) );
    traceBrowserMenu->addAction( vampirAct );
    vampirAct->setEnabled( false );
    vampirAct->setVisible( false );
#endif

#if defined( WITH_PARAVER )
    paraverAct = 0;
    paraverAct = new Action( tr( "Connect to &paraver..." ), this );
    paraverAct->setStatusTip( tr( "Connect to paraver and display a trace file" ) );
    paraverAct->setWhatsThis( "Connect to paraver and display a trace file" );
    connect( paraverAct, SIGNAL( triggered() ), this, SLOT( paraverMenu() ) );
    traceBrowserMenu->addAction( paraverAct );
    paraverAct->setEnabled( false );
    paraverAct->setVisible( false );
#endif
}

void
MainWidget::initializeTraceBrowserMenu( QString const& )
{
    //initialize statistics info
    if ( statistics->existsMaxSeverities() )
    {
        toggleTraceBrowserMenuVisibility( true );
    }
}

void
MainWidget::toggleTraceBrowserMenuVisibility( bool visible )
{
    if ( traceBrowserMenu != 0 )
    {
        traceBrowserMenu->setEnabled( visible );
        traceBrowserMenu->menuAction()->setVisible( visible );
    }
#if defined( WITH_VAMPIR )
    if ( vampirAct != 0 )
    {
        vampirAct->setEnabled( visible );
        vampirAct->setVisible( visible );
    }
#endif
#if defined( WITH_PARAVER )
    if ( paraverAct != 0 )
    {
        paraverAct->setEnabled( visible );
        paraverAct->setVisible( visible );
    }
#endif
}

void
MainWidget::disableTraceBrowserMenu()
{
    if ( statistics != 0 && statistics->existsMaxSeverities() )
    {
        toggleTraceBrowserMenuVisibility( false );
    }
}

#if defined( WITH_VAMPIR )
//slot for connecting to vampir and displaying traces there
void
MainWidget::vampirMenu()
{
    VampirConnectionDialog* dialog = new VampirConnectionDialog( this, fileLine,
                                                                 statistics, this );

    dialog->setModal( false );
    dialog->show();
}
#endif

#define MAKE_STR( x ) _MAKE_STR( x )
#define _MAKE_STR( x ) #x

#if defined( WITH_PARAVER )
void
MainWidget::paraverMenu()
{
    int const spacing = 20;

    QDialog dialog( this );
    dialog.setWindowTitle( "Connect to paraver" );

    QFontMetrics fm( font() );
    int          h = 10 * fm.ascent();
    int          w = 4 * fm.width( "Connect to paraver:" );
    dialog.setMinimumSize( w, h );

    QVBoxLayout* layout = new QVBoxLayout();
    layout->setSpacing( spacing );

    QHBoxLayout* layout1 = new QHBoxLayout();
    layout1->setSpacing( spacing );

    //define configuration file label and editor
    QLabel* configLabel = new QLabel();
    configLabel->setText( "Configuration file:" );
    layout1->addWidget( configLabel );

    configLine = new QLineEdit( MAKE_STR( PV_DEF_CFG ) );
    layout1->addWidget( configLine );
    QPushButton* configFileButton = new QPushButton( "Browse" );
    layout1->addWidget( configFileButton );
    connect( configFileButton, SIGNAL( clicked() ), this, SLOT( getConfigFileName() ) );

    layout->addLayout( layout1 );

    //define file label and editor
    QHBoxLayout* layout2 = new QHBoxLayout();
    layout2->setSpacing( spacing );

    QLabel* fileLabel = new QLabel();
    fileLabel->setText( "Trace file:" );
    layout2->addWidget( fileLabel );

    fileLine = new QLineEdit( statistics->getDefaultParaverFileName().c_str() );
    layout2->addWidget( fileLine );
    QPushButton* fileButton = new QPushButton( "Browse" );
    layout2->addWidget( fileButton );
    connect( fileButton, SIGNAL( clicked() ), this, SLOT( getTraceFileName() ) );

    layout->addLayout( layout2 );

    QDialogButtonBox* buttonBox    = new QDialogButtonBox();
    QPushButton*      okButton     = buttonBox->addButton( QDialogButtonBox::Ok );
    QPushButton*      cancelButton = buttonBox->addButton( QDialogButtonBox::Cancel );
    connect( okButton, SIGNAL( clicked() ), &dialog, SLOT( accept() ) );
    connect( cancelButton, SIGNAL( clicked() ), &dialog, SLOT( reject() ) );
    layout->addWidget( buttonBox );

    dialog.setLayout( layout );

    if ( dialog.exec() )
    {
        assert( statistics != 0 );
        std::string result =
            statistics->connectToParaver( configLine->displayText().toStdString(),
                                          fileLine->displayText().toStdString() );
        if ( result != "" )
        {
            QMessageBox::critical( this, tr( "Paraver connection" ),
                                   QString::fromStdString( result ) );
        }
    }
}
#endif

#undef _MAKE_STR
#undef MAKE_STR

void
MainWidget::getTraceFileName()
{
    QString openFileName =
        QFileDialog::getOpenFileName( this, tr( "Choose a file to open" ),
                                      fileLine->displayText(),
                                      tr( "Trace files (*.elg *.esd *.otf *.otf2 *.prv);;All files (*.*);;All files (*)" ) );
    if ( openFileName.length() == 0 )
    {
        statusBar()->showMessage( tr( "Ready" ) );
        return;
    }
    fileLine->setText( openFileName );
}

#if defined( WITH_PARAVER )
void
MainWidget::getConfigFileName()
{
    QString configFileName = QFileDialog::getOpenFileName( this,
                                                           tr( "Choose a file to open" ), configLine->displayText(),
                                                           tr( "Config files (*.cfg);;All files (*.*);;All files (*)" ) );
    if ( configFileName.length() == 0 )
    {
        statusBar()->showMessage( tr( "Ready" ) );
        return;
    }
    configLine->setText( configFileName );
}
#endif
