/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#include "SystemTopologyWidget.h"

class SystemTopologyToolBar : public QToolBar
{
    Q_OBJECT
public:
    static SystemTopologyToolBar*
    getInstance();
    void
    setParent( SystemTopologyWidget* parent );

    void
    enableTopologyButtons();
    void
    disableTopologyButtons();
    bool
    isFocusEnabled();
    int
    getRecentSelection();
    void
    setRecentSelection( int systemId );
    void
    reinit();

public slots:
    void
    enableFocus( bool );

private slots:
    void
    setButtonsToIcon();
    void
    setButtonsToText();
    void
    moveDown();
    void
    moveUp();
    void
    moveRight();
    void
    moveLeft();
    void
    increasePlaneDistance();
    void
    decreasePlaneDistance();
    void
    zoomIn();
    void
    zoomOut();
    void
    reset();
    void
    scale();
    void
    setXAngle( int angle );
    void
    setYAngle( int angle );

    //get user-defined minimal and maximal values for the minimal and maximal colors
    void
    minMaxValues();

private:
    SystemTopologyToolBar();
    static SystemTopologyToolBar* single;
    SystemTopologyWidget*         parent;
    QAction*                      leftAct, * rightAct, * upAct, * downAct;
    QAction*                      dist1Act, * dist2Act, * zoom1Act, * zoom2Act;
    QAction*                      resetAct, * scaleAct, * userAct;
    QSpinBox*                     xSpin, * ySpin;
    QLabel*                       xLabel, * yLabel;
    bool                          focusEnabled;
    int                           recentSystemId;
};
