/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

// Controller

#ifndef _SYSTEMTOPOLOGYWIDGET_H
#define _SYSTEMTOPOLOGYWIDGET_H

#include <QScrollArea>
#include <vector>

#include "SystemTreeWidget.h"
#include "Cube.h"
#include "SystemTopologyData.h"
#include "SystemTopologyView.h"
#include "SystemTopologyViewTransform.h"
#include "TopologyDimensionBar.h"

class ScrollArea;
class QSettings;

class SystemTopologyWidget : public QScrollArea, public TabInterface
{
    Q_OBJECT

signals:
    void
    xAngleChanged( int angle );
    void
    yAngleChanged( int angle );

private slots:
    void
    vscroll( int val );
    void
    hscroll( int val );
    void
    handleSelection( TreeWidgetItem* );

public slots:
    void
    selectItem( int  systemID,
                bool add );
    void
    scrollTo( int x,
              int y );

public:
    SystemTopologyWidget( SystemTreeWidget* systemTreeWidget,
                          unsigned          topologyId );
    ~SystemTopologyWidget();

    QWidget*
    widget();

    //initialize topology from the cube object
    void
    initialize( cube::Cube* cube );

    //re-compute the colors
    void
    updateColors();

    //remove all data
    void
    cleanUp();

    //rescale the topology sizes such that the topology fits into the scroll widget's window
    void
    rescale();

    // returns the preferred size
    virtual QSize
    sizeHint() const;

    //save and load topology settings
    void
    saveSettings( QSettings& settings );
    bool
    loadSettings( QSettings& settings );

    //update the value widget
    void
    updateValueWidget();

    //set the color for the lines framing processes/threads in the topology
    void
    setLineType( LineType lineType );

    //toggle the flag if unused planes should be painted or not
    void
    toggleUnused();

    QWidget*
    getDimensionSelectionBar( cube::Cube* cube );

    SystemTopologyViewTransform*
    getTransform()
    {
        return transform;
    }
    SystemTopologyData*
    getData()
    {
        return data;
    }
    // TabInterface interface
    QIcon
    icon() const;

protected:
    void
    resizeEvent( QResizeEvent* event );

private:
    typedef enum { SELECT, FOLD } Mode;
    Mode selectMode;                         // if > 2 dimensions: call multiDimFold or multiDimSelect

    SystemTreeWidget*     systemTreeWidget;
    TopologyDimensionBar* dimensionBar;
    SplitterContainer*    tabWidget;

    SystemTopologyData*          data;
    SystemTopologyView*          view;
    SystemTopologyViewTransform* transform;
    unsigned                     topologyId;
    QString                      topologyName;

public:
    // TabInterface implementation
    QString
    label() const;

    void
    valueModusChanged()
    {
    }
    //update the colors and repaint
    void
    valuesChanged();
    void
    setActive();
};

#endif
