/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#ifndef _SYSTEMTREEWIDGET_H
#define _SYSTEMTREEWIDGET_H

#include <QMenu>
#include <QFile>

#include "CubeMetric.h"
#include "CubeRegion.h"
#include "CubeSysres.h"

#include "Constants.h"
#include "TreeWidgetItem.h"
#include "TreeWidget.h"
#include "Action.h"
#include "Coloring.h"
#include "MyTreeWidget.h"
#include "ScrollArea.h"

//the TreeWidget class extends MyTreeWidget basically by value computations

class SystemTreeWidget : public TreeWidget
{
    Q_OBJECT
public:

    SystemTreeWidget( QWidget*            parent,
                      QString             cubeFileName = "",
                      cubeparser::Driver* driver = NULL,
                      Statistics*         statistics = 0 );
    virtual
    ~SystemTreeWidget()
    {
    };

// reimplement the call computeValues


    virtual void
    computeValue( cube::list_of_metrics& mv,
                  TreeWidgetItem*        item );
    virtual void
    computeValue( cube::list_of_metrics& mv,
                  cube::list_of_cnodes&  cv,
                  TreeWidgetItem*        item );
    virtual void
    computeValue( cube::list_of_metrics& mv,
                  cube::list_of_regions& rv,
                  TreeWidgetItem*        item );

    virtual void
    computeBasicValues();

    void
    updateSelection();

    std::vector<TreeWidgetItem*>
    getActiveSubset();
    void
    setSubsetCombo( QComboBox* combo );

    // TabInterface interface
    QIcon
    icon() const;
    QWidget*
    widget();
    virtual void
    setActive();
    void
    valuesChanged();

private slots:
    void
    resetSubsetCombo();
    void
    fillSubsetCombo( const QString& name = "" );
    void
    updateSubsetCombo();

private:
    double
    computeValue( TreeWidgetItem*          metricItem,
                  cube::CalculationFlavour metricState,
                  TreeWidgetItem*          callItem,
                  cube::CalculationFlavour callState,
                  TreeWidgetItem*          systemItem,
                  cube::CalculationFlavour systemState );

    SplitterContainer* container;
    QComboBox*         subsetCombo;
    QStringListModel   subsetModel;

    QString
    getSubsetName( const QString& name = "" );
    int
    getSubsetItemCount( const QString& name );

    std::vector<TreeWidgetItem*>
    getVisitedSubset();
};


#endif
