/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#ifndef TABINTERFACE_H
#define TABINTERFACE_H

#include <QSettings>
#include <QIcon>
#include "Constants.h"
#include "Cube.h"

/**
 * @brief The TabInterface class is inherited from all widgets, which are placed in the
 * SystemTabWidget, CallTabWidget or SystemTabWidget
 */

class TabInterface
{
public:
    virtual QWidget*
    widget() = 0;

    virtual QString
    label() const = 0;

    // values have been changed (eg. item values, colors, font...)  => update display
    virtual void
    valuesChanged() = 0;

    // value modus combo box has been changed
    virtual void
    valueModusChanged() = 0;

    virtual bool
    loadSettings( QSettings& settings ) = 0;

    virtual void
    saveSettings( QSettings& settings ) = 0;

    // ---- end of pure virtual functions -----------------------------------------------

    virtual QIcon
    icon() const
    {
        return QIcon();
    }

    virtual void
    initialize( cube::Cube* )
    {
    }

    // called if current Tab becomes active and visible
    virtual void
    setActive()
    {
    }

    virtual void
    setLineType( LineType )
    {
    }

    // has be be implemented by TreeWidgets, to update values from cube library
    virtual void
    recalculateItems()
    {
    }

    // currently only implemented by TopologyWidgets
    virtual void
    toggleUnused()
    {
    }
};

#endif // TABINTERFACE_H
