/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#include "MainWidget.h"
#include "CubeError.h"
#include <iostream>
#include <iomanip>
#include <QFileDialog>
#include <QApplication>

using namespace std;

//this is the main method
//handles command-line arguments and starts the GUI

int
main( int argc, char* argv[] )
{
//    setlocale(LC_ALL,"");
    setlocale( LC_ALL, "" );
    setlocale( LC_CTYPE, "C" );
    QApplication app( argc, argv );

    if ( argc > 2 )
    {
        std::cout << "Usage: cube [filename]\n";
        return 0;
    }
    try
    {
        MainWidget mainWidget;
        mainWidget.show();
        if ( argc == 2 )
        {
            QString name = QString::fromUtf8( argv[ 1 ] );
            //if the command line argument is a directory then
            //open a file dialog for getting the file name
            if ( !QFile( name ).exists() || QDir( name ).exists() )
            {
                name = QFileDialog::getOpenFileName( NULL,
                                                     app.tr( "Choose a file to open" ),
                                                     name,
                                                     app.tr( "Cube3/4 files (*cube *cube.gz *.cubex);;Cube4 files (*.cubex);;Cube files (*.cube.gz *.cube);;All files (*.*);;All files (*)" ) );
            }
            if ( name.length() > 0 )
            {
                mainWidget.readFile( name );
            }
        }
        return app.exec();
    }
    catch ( cube::RuntimeError err )
    {
        std::cerr << "ERROR in main: " << err.get_msg().c_str() << std::endl;
    }
    return -1;
}
