/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
/**
 *
 * \file CubeSignedValue.cpp
 * \brief  Defines the methods of the "SignedValue".
 ************************************************/

#ifndef __SIGNED_VALUE_CPP
#define __SIGNED_VALUE_CPP

#include <sstream>
#include <cstring>
#include <string>
#include "CubeValues.h"
using namespace std;
using namespace cube;



SignedValue::SignedValue()
{
    isSingleValue = true;
    value.iValue  = 0;
}

// SignedValue::SignedValue( uint16_t uv )
// {
//     isSingleValue = true;
//
//     value.iValue = ( int32_t )uv;
// }
// SignedValue::SignedValue( int16_t uv )
// {
//     isSingleValue = true;
//
//     value.iValue = ( int32_t )uv;
// }
//
// SignedValue::SignedValue( uint32_t uv )
// {
//     isSingleValue = true;
//
//     value.iValue = ( int32_t )uv;
// }
SignedValue::SignedValue( int32_t uv )
{
    isSingleValue = true;

    value.iValue = uv;
}
// // SignedValue::SignedValue( uint64_t uv )
// // {
// //     isSingleValue = true;
// //
// //     value.iValue = ( int32_t )uv;
// // }
// // SignedValue::SignedValue( int64_t uv )
// // {
// //     isSingleValue = true;
// //
// //     value.iValue = ( int32_t )uv;
// // }
// //
// //
// // SignedValue::SignedValue( double dv )
// // {
// //     isSingleValue = true;
// //
// //     value.iValue = ( int32_t )dv;
// // }
// // SignedValue::SignedValue( char* cv )
// // {
// //     isSingleValue = true;
// //
// //     memcpy( value.aValue, cv,  sizeof( int32_t ) );
// // }
// //
// // unsigned
// // SignedValue::getSize()
// // {
// //     return sizeof( int32_t );
// // }
// // double
// // SignedValue::getDouble()
// // {
// //     return ( double )value.iValue;
// // }
// //
// //

uint16_t
SignedValue::getUnsignedShort()
{
    return ( uint16_t )value.iValue;
}
int16_t
SignedValue::getSignedShort()
{
    return ( int16_t )value.iValue;
}

uint32_t
SignedValue::getUnsignedInt()
{
    return ( uint32_t )value.iValue;
}
int32_t
SignedValue::getSignedInt()
{
    return value.iValue;
}


uint64_t
SignedValue::getUnsignedLong()
{
    return ( uint64_t )value.iValue;
}
int64_t
SignedValue::getSignedLong()
{
    return ( int64_t )value.iValue;
}



/**
 * As char will be returned just first char of the char representation
 */
char
SignedValue::getChar()
{
    return ( char )value.iValue;
}

/**
 * Creates the string representation of the value.
 */
string
SignedValue::getString()
{
    stringstream sstr;
    string       str;
    sstr << value.iValue;
    sstr >> str;
    return str;
}
// // /*
// // /**
// //  * Creates the copy and sets the value to 0.
// //  */
// // Value*
// // SignedValue::clone()
// // {
// //     return new SignedValue( 0 );
// // }
// //
// // /**
// //  * Creates the copy.
// //  */
// // Value*
// // SignedValue::copy()
// // {
// //     return new SignedValue( value.iValue );
// // }*/

/**
 * Sets the value from stream and returns the position in stream right after the value.
 */
char*
SignedValue::fromStream( char* cv )
{
    memcpy( value.aValue, cv, sizeof( int32_t ) );
    return cv + sizeof( int32_t );
}


/**
 * Saves the value in the stream and returns the position in stream right after the value.
 */
char*
SignedValue::toStream( char* cv )
{
    memcpy( cv, value.aValue, sizeof( int32_t ) );
    return cv + sizeof( int32_t );
}

/*
   SignedValue
   SignedValue::operator+( const SignedValue& ch )
   {
    SignedValue tmp = *this;
    tmp.value.iValue += ch.value.iValue;
    return tmp;
   }

   SignedValue
   SignedValue::operator-( const SignedValue& ch )
   {
    SignedValue tmp = *this;
    tmp.value.iValue -= ch.value.iValue;
    return tmp;
   }


   void
   SignedValue::operator+=( Value* chval )
   {
    if ( chval == NULL )
    {
        return;
    }
    value.iValue += ( ( SignedValue* )chval )->value.iValue;
   }

   void
   SignedValue::operator-=( Value* chval )
   {
    if ( chval == NULL )
    {
        return;
    }
    value.iValue -= ( ( SignedValue* )chval )->value.iValue;
   }


   void
   SignedValue::operator*=( double dval )
   {
    double _d = ( double )value.iValue;
    _d          *= dval;
    value.iValue = ( int32_t )_d;
   }

   void
   SignedValue::operator/=( double dval )
   {
    if ( dval == 0. )
    {
        cerr << "ERROR: DEVISION BY ZERO!" << endl;
    }
    double _d = ( double )value.iValue;
    _d           /= dval;
    value.iValue /= ( int32_t )_d;
   }
 */

// overloaded new operator
void*
SignedValue::operator new( size_t size )
{
    return ( void* )int32_preallocator.Get();
}
// delete operator overloaded
void
SignedValue::operator delete( void* p )
{
    int32_preallocator.Put( ( SignedValue* )p );
}

void
SignedValue::operator=( double d )
{
// #warning LOST PRECISION
    value.iValue = ( int32_t )d;
}
/*
   void
   SignedValue::operator=( char c )
   {
    value.iValue = ( int32_t )c;
   }


   void
   SignedValue::operator=( uint16_t us )
   {
    value.iValue = ( int32_t )us;
   }

   void
   SignedValue::operator=( uint32_t i )
   {
   // #warning LOST PRECISION
    value.iValue = ( int32_t )i;
   }

   void
   SignedValue::operator=( uint64_t ul )
   {
   // #warning LOST PRECISION
    value.iValue = ( int32_t )ul;
   }

   void
   SignedValue::operator=( int16_t s )
   {
    value.iValue = ( int32_t )s;
   }

   void
   SignedValue::operator=( int32_t i )
   {
    value.iValue = ( int32_t )i;
   }

   void
   SignedValue::operator=( int64_t sl )
   {
   // #warning LOST PRECISION
    value.iValue = ( int32_t )sl;
   }

   void
   SignedValue::operator=( string v )
   {
    stringstream sstr;
    sstr << v;
    sstr >> value.iValue;
   }



   SignedValue
   SignedValue::operator=( SignedValue v )
   {
   // #warning MISSED INTERFACE FOR LONG LONG
    if ( &v == this )
    {
        return *this;
    }
    value.iValue = v.getSignedInt();
    return *this;
   }
 */
void
SignedValue::operator=( Value* v )
{
    value.iValue = v->getSignedInt();
}

void
SignedValue::normalizeWithClusterCount( uint64_t N )
{
    value.iValue = ( int32_t )( ( double )value.iValue / ( double )N );
}

#endif
