/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
/**
 *
 * \file CubeUnsignedLongValue.cpp
 * \brief  Defines the methods of the "UnsignedLongValue".
 ************************************************/

#ifndef __UNSIGNED_LONG_VALUE_CPP
#define __UNSIGNED_LONG_VALUE_CPP

#include <sstream>
#include <cstring>
#include <string>
#include "CubeValues.h"

using namespace std;
using namespace cube;


UnsignedLongValue::UnsignedLongValue()
{
    isSingleValue = true;
    value.ulValue = 0;
}
// UnsignedLongValue::UnsignedLongValue( uint16_t uv )
// {
//     isSingleValue = true;
//     value.ulValue = ( uint64_t )uv;
// }
// UnsignedLongValue::UnsignedLongValue( int16_t uv )
// {
//     isSingleValue = true;
//     value.ulValue = ( uint64_t )uv;
// }
//
// UnsignedLongValue::UnsignedLongValue( uint32_t uv )
// {
//     isSingleValue = true;
//     value.ulValue = ( uint64_t )uv;
// }
// UnsignedLongValue::UnsignedLongValue( int32_t uv )
// {
//     isSingleValue = true;
//     value.ulValue = ( uint64_t )uv;
// }
//

UnsignedLongValue::UnsignedLongValue( uint64_t uv )
{
    isSingleValue = true;
    value.ulValue = ( uint64_t )uv;
}
// UnsignedLongValue::UnsignedLongValue( int64_t uv )
// {
//     isSingleValue = true;
//     value.ulValue = ( uint64_t )uv;
// }




/*

   UnsignedLongValue::UnsignedLongValue( double dv )
   {
    isSingleValue = true;
    value.ulValue = ( uint64_t )dv;
   }
   UnsignedLongValue::UnsignedLongValue( char* cv )
   {
    isSingleValue = true;
    memcpy( value.aValue, cv,  sizeof( uint64_t ) );
   }*/
// /*
// unsigned
// UnsignedLongValue::getSize()
// {
//     return sizeof( uint64_t );
// }
// double
// UnsignedLongValue::getDouble()
// {
//     return ( double )value.ulValue;
// }
//

uint16_t
UnsignedLongValue::getUnsignedShort()
{
    return ( uint16_t )value.ulValue;
}
int16_t
UnsignedLongValue::getSignedShort()
{
    return ( int16_t )value.ulValue;
}

uint32_t
UnsignedLongValue::getUnsignedInt()
{
    return ( uint32_t )value.ulValue;
}
int32_t
UnsignedLongValue::getSignedInt()
{
    return ( int32_t )value.ulValue;
}

uint64_t
UnsignedLongValue::getUnsignedLong()
{
    return ( uint64_t )value.ulValue;
}
int64_t
UnsignedLongValue::getSignedLong()
{
    return ( int64_t )value.ulValue;
}



/**
 * As char will be returned just first char of the char representation
 */
char
UnsignedLongValue::getChar()
{
    return value.aValue[ 0 ];
}

/**
 * Creates the string representation of the value.
 */
string
UnsignedLongValue::getString()
{
    stringstream sstr;
    string       str;
    sstr << value.ulValue;
    sstr >> str;
    return str;
}

// // /*
// // /**
// //  * Creates the copy and sets the value to 0.
// //  */
// // Value*
// // UnsignedLongValue::clone()
// // {
// //     return new UnsignedLongValue( 0 );
// // }
// //
// //
// // /**
// //  * Creates the copy. Missing constructor for unsignel long. Has to be added.
// //  */
// // Value*
// // UnsignedLongValue::copy()
// // {
// // // #warning "lost precision!"
// //     return new UnsignedLongValue( value.ulValue );
// // }*/

/**
 * Sets the value from stream and returns the position in stream right after the value.
 */
char*
UnsignedLongValue::fromStream( char* cv )
{
    memcpy( value.aValue, cv, sizeof( uint64_t ) );
    return cv + sizeof( uint64_t );
}


/**
 * Saves the value in the stream and returns the position in stream right after the value.
 */
char*
UnsignedLongValue::toStream( char* cv )
{
    memcpy( cv, value.aValue, sizeof( uint64_t ) );
    return cv + sizeof( uint64_t );
}

/*
   UnsignedLongValue
   UnsignedLongValue::operator+( const UnsignedLongValue& ch )
   {
    UnsignedLongValue tmp = *this;
    tmp.value.ulValue += ch.value.ulValue;
    return tmp;
   }

   UnsignedLongValue
   UnsignedLongValue::operator-( const UnsignedLongValue& ch )
   {
    UnsignedLongValue tmp = *this;
    tmp.value.ulValue -= ch.value.ulValue;
    return tmp;
   }


   void
   UnsignedLongValue::operator+=( Value* chval )
   {
    if ( chval == NULL )
    {
        return;
    }
    value.ulValue += ( ( UnsignedLongValue* )chval )->value.ulValue;
   }

   void
   UnsignedLongValue::operator-=( Value* chval )
   {
    if ( chval == NULL )
    {
        return;
    }
    value.ulValue -= ( ( UnsignedLongValue* )chval )->value.ulValue;
   }

   void
   UnsignedLongValue::operator*=( double dval )
   {
    double _d = ( double )value.ulValue;
    _d           *= dval;
    value.ulValue = ( uint64_t )_d;
   }

   void
   UnsignedLongValue::operator/=( double dval )
   {
    if ( dval == 0. )
    {
        cerr << "ERROR: DEVISION BY ZERO!" << endl;
    }
    double _d = ( double )value.ulValue;
    _d            /= dval;
    value.ulValue /= ( uint64_t )_d;
   }




 */




// overloaded new operator
void*
UnsignedLongValue::operator new( size_t size )
{
    return ( void* )uint64_preallocator.Get();
}
// delete operator overloaded
void
UnsignedLongValue::operator delete( void* p )
{
    uint64_preallocator.Put( ( UnsignedLongValue* )p );
}

void
UnsignedLongValue::operator=( double d )
{
    value.ulValue = ( uint64_t )d;
}
/*
   void
   UnsignedLongValue::operator=( char c )
   {
    value.ulValue = ( uint64_t )c;
   }


   void
   UnsignedLongValue::operator=( uint16_t ui )
   {
    value.ulValue = ( uint64_t )ui;
   }

   void
   UnsignedLongValue::operator=( uint32_t i )
   {
    value.ulValue = ( uint64_t )i;
   }

   void
   UnsignedLongValue::operator=( uint64_t ul )
   {
    value.ulValue = ( uint64_t )ul;
   }

   void
   UnsignedLongValue::operator=( int16_t si )
   {
    value.ulValue = ( uint64_t )si;
   }

   void
   UnsignedLongValue::operator=( int32_t i )
   {
    value.ulValue = ( uint32_t )i;
   }

   void
   UnsignedLongValue::operator=( int64_t sl )
   {
    value.ulValue = ( uint64_t )sl;
   }

   void
   UnsignedLongValue::operator=( string v )
   {
    stringstream sstr;
    sstr << v;
    sstr >> value.ulValue;
   }

   UnsignedLongValue
   UnsignedLongValue::operator=( UnsignedLongValue v )
   {
    if ( &v == this )
    {
        return *this;
    }
    value.ulValue = v.getUnsignedLong();
    return *this;
   }
 */

void
UnsignedLongValue::operator=( Value* v )
{
    value.ulValue = v->getUnsignedLong();
}

void
UnsignedLongValue::normalizeWithClusterCount( uint64_t N )
{
    value.ulValue = ( uint64_t )( ( double )value.ulValue / ( double )N );
}

#endif
