/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#ifndef __POWER_EVALUATION_CPP
#define __POWER_EVALUATION_CPP 0

#include "CubePowerEvaluation.h"
using namespace cube;

PowerEvaluation::PowerEvaluation() : BinaryEvaluation()
{
};

PowerEvaluation::PowerEvaluation( GeneralEvaluation* _arg1,
                                  GeneralEvaluation* _arg2 ) : BinaryEvaluation( _arg1, _arg2 )
{
};


PowerEvaluation::~PowerEvaluation()
{
}

double
PowerEvaluation::eval()
{
    if ( getNumOfParameters() == 2 )
    {
        double base  = arguments[ 0 ]->eval();
        double power = arguments[ 1 ]->eval();
        return pow( base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;
}


/*

   double
   PowerEvaluation::eval( cube::Cnode * _cnode, CalculationFlavour _cf, cube::Thread * _thread, CalculationFlavour _tf)
   {
    if (getNumOfParameters() == 2)
    {
       double base = arguments[0]->eval(	_cnode, _cf,
                                            _thread, _tf);
        double power = arguments[1]->eval(	_cnode, _cf,
                                            _thread, _tf);
            return pow(base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;

   }



   double PowerEvaluation::eval( cube::Cnode * _cnode, CalculationFlavour _cf, cube::Thread * _th1, cube::Thread * _th2, CalculationFlavour  _tf )
   {

    if (getNumOfParameters() == 2)
    {
        double base = arguments[0]->eval(	_cnode, _cf,
                                            _th1, _th2, _tf);
        double power = arguments[1]->eval(	_cnode, _cf,
                                            _th1, _th2, _tf);
            return pow(base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;
   }
 */

double
PowerEvaluation::eval( cube::Cnode* _cnode, CalculationFlavour _cf, cube::Sysres* _sf, CalculationFlavour _tf   )
{
    if ( getNumOfParameters() == 2 )
    {
        double base = arguments[ 0 ]->eval(       _cnode, _cf,
                                                  _sf, _tf );
        double power = arguments[ 1 ]->eval(      _cnode, _cf,
                                                  _sf, _tf );
        return pow( base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;
}
/*
   double
   PowerEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Process * _pr, CalculationFlavour  _tf   )
   {

    if (getNumOfParameters() == 2)
    {
        double base = arguments[0]->eval(		_cnode, _cf,
                                        _pr, _tf);
        double power = arguments[1]->eval(		_cnode, _cf,
                                        _pr, _tf);
            return pow(base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;
   }

   double
   PowerEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Node * _nd, CalculationFlavour    _tf )
   {

    if (getNumOfParameters() == 2)
    {
        double base = arguments[0]->eval(	_cnode, _cf,
                                            _nd, _tf);
        double power = arguments[1]->eval(	_cnode, _cf,
                                            _nd, _tf);
            return pow(base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;

   }

   double
   PowerEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Machine * _mch, CalculationFlavour   _tf  )
   {

    if (getNumOfParameters() == 2)
    {
        double base = arguments[0]->eval(	_cnode, _cf,
                                            _mch, _tf);
        double power = arguments[1]->eval(	_cnode, _cf,
                                            _mch, _tf);
            return pow(base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;
   }*/

double
PowerEvaluation::eval( cube::Cnode* _cnode, CalculationFlavour _cf )
{
    if ( getNumOfParameters() == 2 )
    {
        double base  = arguments[ 0 ]->eval(       _cnode, _cf );
        double power = arguments[ 1 ]->eval(      _cnode, _cf );
        return pow( base, power );
    }
    cerr << " Power is not defined for else  than two paraments." << endl;
    return 0.;
}



#endif
