/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#ifndef __NEGATIVE_EVALUATION_CPP
#define __NEGATIVE_EVALUATION_CPP 0

#include "CubeNegativeEvaluation.h"
using namespace cube;
NegativeEvaluation::NegativeEvaluation() : UnaryEvaluation()
{
}

NegativeEvaluation::NegativeEvaluation( GeneralEvaluation* _arg ) : UnaryEvaluation( _arg )
{
}


NegativeEvaluation::~NegativeEvaluation()
{
}

double
NegativeEvaluation::eval()
{
    if ( getNumOfParameters() == 0 )
    {
        return 0;
    }
    if ( getNumOfParameters() == 1 )
    {
        return -( arguments[ 0 ]->eval() );
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;
}

/*
   double
   NegativeEvaluation::eval( Cnode * _cnode, CalculationFlavour _cf , Thread * _th, CalculationFlavour _tf)
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 1)
    {
        return -(arguments[0]->eval(_cnode, _cf, _th, _tf));
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;
   }

   double
   NegativeEvaluation::eval( Cnode *  _cnode, CalculationFlavour _cf , Thread * _th1, Thread * _th2, CalculationFlavour   _tf )
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 1)
    {
        return -(arguments[0]->eval( _cnode, _cf, _th1, _th2, _tf));
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;

   }
 */
double
NegativeEvaluation::eval( Cnode* _cnode, CalculationFlavour _cf, Sysres* _sys, CalculationFlavour _sf  )
{
    if ( getNumOfParameters() == 0 )
    {
        return 0;
    }
    if ( getNumOfParameters() == 1 )
    {
        return -( arguments[ 0 ]->eval( _cnode, _cf, _sys, _sf ) );
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;
}
/*
   double
   NegativeEvaluation::eval( Cnode *  _cnode, CalculationFlavour _cf , Process * _pc, CalculationFlavour _pf  )
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 1)
    {
        return -(arguments[0]->eval( _cnode, _cf, _pc, _pf));
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;

   }

   double
   NegativeEvaluation::eval( Cnode *  _cnode, CalculationFlavour _cf , Node * _nd, CalculationFlavour  _nf)
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 1)
    {
        return -(arguments[0]->eval( _cnode, _cf, _nd, _nf));
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;

   }

   double
   NegativeEvaluation::eval( Cnode *  _cnode, CalculationFlavour _cf , Machine * _mach, CalculationFlavour _mf  )
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 1)
    {
        return -(arguments[0]->eval( _cnode, _cf, _mach, _mf));
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;

   }*/

double
NegativeEvaluation::eval( Cnode* _cnode, CalculationFlavour _cf )
{
    if ( getNumOfParameters() == 0 )
    {
        return 0;
    }
    if ( getNumOfParameters() == 1 )
    {
        return -( arguments[ 0 ]->eval( _cnode, _cf ) );
    }
    cerr << " Negation of more than 1 parameters is not defined" << endl;
    return 0;
}



#endif
