/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#ifndef __LAMBDA_CALCUL_EVALUATION_CPP
#define __LAMBDA_CALCUL_EVALUATION_CPP 0

#include <cmath>

#include "CubeLambdaCalculEvaluation.h"
using namespace cube;
LambdaCalculEvaluation::LambdaCalculEvaluation() : UnaryEvaluation()
{
};


LambdaCalculEvaluation::~LambdaCalculEvaluation()
{
}

double
LambdaCalculEvaluation::eval()
{
    if ( getNumOfParameters() == 0 )
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[ i ]->eval();
    }
    return arguments[ getNumOfParameters() - 1 ]->eval();
}

/*

   double
   LambdaCalculEvaluation::eval( cube::Cnode * _cnode, CalculationFlavour _cf, cube::Thread * _thread, CalculationFlavour _tf)
   {
    if (getNumOfParameters()==0)
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[i]->eval(_cnode, _cf, _thread, _tf);
    }
    return arguments[ getNumOfParameters() - 1 ]->eval( _cnode, _cf, _thread, _tf );
   }



   double LambdaCalculEvaluation::eval( cube::Cnode * _cnode, CalculationFlavour _cf, cube::Thread * _th1, cube::Thread * _th2, CalculationFlavour  _tf )
   {

    if (getNumOfParameters()==0)
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[i]->eval(_cnode, _cf, _th1, _th2, _tf);
    }
    return arguments[ getNumOfParameters() - 1 ]->eval( _cnode, _cf, _th1, _th2, _tf );
   }
 */

double
LambdaCalculEvaluation::eval( cube::Cnode* _cnode, CalculationFlavour _cf, cube::Sysres* _sf, CalculationFlavour _tf   )
{
    if ( getNumOfParameters() == 0 )
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[ i ]->eval( _cnode, _cf, _sf, _tf );
    }
    return arguments[ getNumOfParameters() - 1 ]->eval( _cnode, _cf, _sf, _tf );
}

/*
   double
   LambdaCalculEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Process * _pr, CalculationFlavour  _tf   )
   {
    if (getNumOfParameters()==0)
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[i]->eval(_cnode, _cf, _pr, _tf);
    }
    return arguments[ getNumOfParameters() - 1 ]->eval( _cnode, _cf, _pr, _tf );
   }

   double
   LambdaCalculEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Node * _nd, CalculationFlavour    _tf )
   {
    if (getNumOfParameters()==0)
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[i]->eval(_cnode, _cf, _nd, _tf);
    }
    return arguments[ getNumOfParameters() - 1 ]->eval( _cnode, _cf, _nd, _tf );
   }

   double
   LambdaCalculEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Machine * _mch, CalculationFlavour   _tf  )
   {
    if (getNumOfParameters()==0)
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[i]->eval(_cnode, _cf, _mch, _tf);
    }
    return arguments[ getNumOfParameters() - 1 ]->eval( _cnode, _cf, _mch, _tf );
   }*/

double
LambdaCalculEvaluation::eval( cube::Cnode* _cnode, CalculationFlavour _cf )
{
    if ( getNumOfParameters() == 0 )
    {
        cerr << " Lambda calcul needs at leatst one return expression. " << endl;
        return 0.;
    }
    for ( unsigned i = 0; i < getNumOfParameters() - 1; i++ )
    {
        arguments[ i ]->eval( _cnode, _cf );
    }
    return arguments[ getNumOfParameters() - 1 ]->eval( _cnode, _cf );
}




#endif
