/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
#ifndef __MAX_EVALUATION_CPP
#define __MAX_EVALUATION_CPP 0

#include <cmath>
#include <algorithm>

#include "CubeMaxEvaluation.h"
using namespace cube;
MaxEvaluation::MaxEvaluation() : UnaryEvaluation()
{
};


MaxEvaluation::~MaxEvaluation()
{
}

double
MaxEvaluation::eval()
{
    if ( getNumOfParameters() == 0 )
    {
        return 0;
    }
    if ( getNumOfParameters() == 2 )
    {
        return std::max( arguments[ 0 ]->eval(), arguments[ 1 ]->eval() );
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0.;
}


/*

   double
   MaxEvaluation::eval( cube::Cnode * _cnode, CalculationFlavour _cf, cube::Thread * _thread, CalculationFlavour _tf)
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 2)
    {
        double _value = std::max(arguments[0]->eval(	_cnode, _cf,
                                            _thread, _tf
                            ),
                            arguments[1]->eval(	_cnode, _cf,
                                            _thread, _tf
                            )
                            );
        return _value;
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0;
   }



   double MaxEvaluation::eval( cube::Cnode * _cnode, CalculationFlavour _cf, cube::Thread * _th1, cube::Thread * _th2, CalculationFlavour  _tf )
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 2)
    {
        double _value = std::max(arguments[0]->eval(	_cnode, _cf,
                                            _th1, _th2, _tf
                            ),
                            arguments[1]->eval(	_cnode, _cf,
                                            _th1, _th2, _tf
                            )
                            );
        return _value;
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0;
   }
 */

double
MaxEvaluation::eval( cube::Cnode* _cnode, CalculationFlavour _cf, cube::Sysres* _sf, CalculationFlavour _tf   )
{
    if ( getNumOfParameters() == 0 )
    {
        return 0;
    }
    if ( getNumOfParameters() == 2 )
    {
        double _value = std::max( arguments[ 0 ]->eval(    _cnode, _cf,
                                                           _sf, _tf
                                                           ),
                                  arguments[ 1 ]->eval( _cnode, _cf,
                                                        _sf, _tf
                                                        )
                                  );
        return _value;
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0;
}
/*
   double
   MaxEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Process * _pr, CalculationFlavour  _tf   )
   {
    if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 2)
    {
        double _value = std::max(arguments[0]->eval(	_cnode, _cf,
                                        _pr, _tf
                            ),
                            arguments[1]->eval(	_cnode, _cf,
                                        _pr, _tf
                            )
                            );
        return _value;
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0;
   }

   double
   MaxEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Node * _nd, CalculationFlavour    _tf )
   {
     if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 2)
    {
        double _value = std::max(arguments[0]->eval(	_cnode, _cf,
                                            _nd, _tf
                                ),
                                arguments[1]->eval(	_cnode, _cf,
                                            _nd, _tf
                                )
                                );
        return _value;
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0;
   }

   double
   MaxEvaluation::eval( cube::Cnode *  _cnode, CalculationFlavour _cf, cube::Machine * _mch, CalculationFlavour   _tf  )
   {
     if (getNumOfParameters() == 0 )
        return 0;
    if ( getNumOfParameters() == 2)
    {
        double _value = std::max(arguments[0]->eval(	_cnode, _cf,
                                            _mch, _tf
                        ),
                        arguments[1]->eval(	_cnode, _cf,
                                            _mch, _tf
                        )
                        );
        return _value;
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0;
   }*/

double
MaxEvaluation::eval( cube::Cnode* _cnode, CalculationFlavour _cf )
{
    if ( getNumOfParameters() == 0 )
    {
        return 0;
    }
    if ( getNumOfParameters() == 2 )
    {
        double _value = std::max( arguments[ 0 ]->eval(    _cnode, _cf
                                                           ),
                                  arguments[ 1 ]->eval(     _cnode, _cf
                                                            )
                                  );
        return _value;
    }
    cerr << " Max of else than 2 parameters is not defined" << endl;
    return 0;
}




#endif
